@@ -20,6 +20,7 @@ package xdsresource
 import (
 	"errors"
 	"fmt"
+	"math"
 	"regexp"
 	"testing"
 	"time"
@@ -30,14 +31,14 @@ import (
 	"github.com/google/go-cmp/cmp/cmpopts"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/internal/envconfig"
+	"google.golang.org/grpc/internal/pretty"
 	"google.golang.org/grpc/internal/testutils"
+	"google.golang.org/grpc/internal/xds/matcher"
 	"google.golang.org/grpc/xds/internal/clusterspecifier"
 	"google.golang.org/grpc/xds/internal/httpfilter"
 	"google.golang.org/grpc/xds/internal/xdsclient/xdsresource/version"
 	"google.golang.org/protobuf/types/known/durationpb"
 
-	v2xdspb "github.com/envoyproxy/go-control-plane/envoy/api/v2"
-	v2routepb "github.com/envoyproxy/go-control-plane/envoy/api/v2/route"
 	v3corepb "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
 	rpb "github.com/envoyproxy/go-control-plane/envoy/config/rbac/v3"
 	v3routepb "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
@@ -412,38 +413,6 @@ func (s) TestRDSGenerateRDSUpdateFromRouteConfiguration(t *testing.T) {
 				},
 			},
 		},
-		{
-			// weights not add up to total-weight.
-			name: "route-config-with-weighted_clusters_weights_not_add_up",
-			rc: &v3routepb.RouteConfiguration{
-				Name: routeName,
-				VirtualHosts: []*v3routepb.VirtualHost{
-					{
-						Domains: []string{ldsTarget},
-						Routes: []*v3routepb.Route{
-							{
-								Match: &v3routepb.RouteMatch{PathSpecifier: &v3routepb.RouteMatch_Prefix{Prefix: "/"}},
-								Action: &v3routepb.Route_Route{
-									Route: &v3routepb.RouteAction{
-										ClusterSpecifier: &v3routepb.RouteAction_WeightedClusters{
-											WeightedClusters: &v3routepb.WeightedCluster{
-												Clusters: []*v3routepb.WeightedCluster_ClusterWeight{
-													{Name: "a", Weight: &wrapperspb.UInt32Value{Value: 2}},
-													{Name: "b", Weight: &wrapperspb.UInt32Value{Value: 3}},
-													{Name: "c", Weight: &wrapperspb.UInt32Value{Value: 5}},
-												},
-												TotalWeight: &wrapperspb.UInt32Value{Value: 30},
-											},
-										},
-									},
-								},
-							},
-						},
-					},
-				},
-			},
-			wantError: true,
-		},
 		{
 			name: "good-route-config-with-weighted_clusters",
 			rc: &v3routepb.RouteConfiguration{
@@ -463,7 +432,6 @@ func (s) TestRDSGenerateRDSUpdateFromRouteConfiguration(t *testing.T) {
 													{Name: "b", Weight: &wrapperspb.UInt32Value{Value: 3}},
 													{Name: "c", Weight: &wrapperspb.UInt32Value{Value: 5}},
 												},
-												TotalWeight: &wrapperspb.UInt32Value{Value: 10},
 											},
 										},
 									},
@@ -738,7 +706,7 @@ func (s) TestRDSGenerateRDSUpdateFromRouteConfiguration(t *testing.T) {
 			rlsEnabled: true,
 		},
 		{
-			name: "ignore-error-in-cluster-specifier-plugin",
+			name: "ignore-error-in-cluster-specifier-plugin-env-var-off",
 			rc: goodRouteConfigWithClusterSpecifierPlugins([]*v3routepb.ClusterSpecifierPlugin{
 				clusterSpecifierPlugin("cspA", configOfClusterSpecifierDoesntExist, false),
 			}, []string{}),
@@ -749,7 +717,7 @@ func (s) TestRDSGenerateRDSUpdateFromRouteConfiguration(t *testing.T) {
 			rc: goodRouteConfigWithClusterSpecifierPlugins([]*v3routepb.ClusterSpecifierPlugin{
 				clusterSpecifierPlugin("cspA", mockClusterSpecifierConfig, false),
 			}, []string{"cspA"}),
-			wantError: true,
+			wantUpdate: goodUpdate,
 		},
 		// This tests a scenario where a cluster specifier plugin is not found
 		// and is optional. Any routes referencing that not found optional
@@ -777,7 +745,7 @@ func (s) TestRDSGenerateRDSUpdateFromRouteConfiguration(t *testing.T) {
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
 			envconfig.XDSRLS = test.rlsEnabled
-			gotUpdate, gotError := generateRDSUpdateFromRouteConfiguration(test.rc, nil, false)
+			gotUpdate, gotError := generateRDSUpdateFromRouteConfiguration(test.rc)
 			if (gotError != nil) != test.wantError ||
 				!cmp.Equal(gotUpdate, test.wantUpdate, cmpopts.EquateEmpty(),
 					cmp.Transformer("FilterConfig", func(fc httpfilter.FilterConfig) string {
@@ -835,45 +803,11 @@ func (s) TestUnmarshalRouteConfig(t *testing.T) {
 		ldsTarget                = "lds.target.good:1111"
 		uninterestingDomain      = "uninteresting.domain"
 		uninterestingClusterName = "uninterestingClusterName"
-		v2RouteConfigName        = "v2RouteConfig"
 		v3RouteConfigName        = "v3RouteConfig"
-		v2ClusterName            = "v2Cluster"
 		v3ClusterName            = "v3Cluster"
 	)
 
 	var (
-		v2VirtualHost = []*v2routepb.VirtualHost{
-			{
-				Domains: []string{uninterestingDomain},
-				Routes: []*v2routepb.Route{
-					{
-						Match: &v2routepb.RouteMatch{PathSpecifier: &v2routepb.RouteMatch_Prefix{Prefix: ""}},
-						Action: &v2routepb.Route_Route{
-							Route: &v2routepb.RouteAction{
-								ClusterSpecifier: &v2routepb.RouteAction_Cluster{Cluster: uninterestingClusterName},
-							},
-						},
-					},
-				},
-			},
-			{
-				Domains: []string{ldsTarget},
-				Routes: []*v2routepb.Route{
-					{
-						Match: &v2routepb.RouteMatch{PathSpecifier: &v2routepb.RouteMatch_Prefix{Prefix: ""}},
-						Action: &v2routepb.Route_Route{
-							Route: &v2routepb.RouteAction{
-								ClusterSpecifier: &v2routepb.RouteAction_Cluster{Cluster: v2ClusterName},
-							},
-						},
-					},
-				},
-			},
-		}
-		v2RouteConfig = testutils.MarshalAny(&v2xdspb.RouteConfiguration{
-			Name:         v2RouteConfigName,
-			VirtualHosts: v2VirtualHost,
-		})
 		v3VirtualHost = []*v3routepb.VirtualHost{
 			{
 				Domains: []string{uninterestingDomain},
@@ -907,288 +841,90 @@ func (s) TestUnmarshalRouteConfig(t *testing.T) {
 			VirtualHosts: v3VirtualHost,
 		})
 	)
-	const testVersion = "test-version-rds"
 
 	tests := []struct {
 		name       string
-		resources  []*anypb.Any
-		wantUpdate map[string]RouteConfigUpdateErrTuple
-		wantMD     UpdateMetadata
+		resource   *anypb.Any
+		wantName   string
+		wantUpdate RouteConfigUpdate
 		wantErr    bool
 	}{
 		{
-			name:      "non-routeConfig resource type",
-			resources: []*anypb.Any{{TypeUrl: version.V3HTTPConnManagerURL}},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusNACKed,
-				Version: testVersion,
-				ErrState: &UpdateErrorMetadata{
-					Version: testVersion,
-					Err:     cmpopts.AnyError,
-				},
-			},
-			wantErr: true,
+			name:     "non-routeConfig resource type",
+			resource: &anypb.Any{TypeUrl: version.V3HTTPConnManagerURL},
+			wantErr:  true,
 		},
 		{
 			name: "badly marshaled routeconfig resource",
-			resources: []*anypb.Any{
-				{
-					TypeUrl: version.V3RouteConfigURL,
-					Value:   []byte{1, 2, 3, 4},
-				},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusNACKed,
-				Version: testVersion,
-				ErrState: &UpdateErrorMetadata{
-					Version: testVersion,
-					Err:     cmpopts.AnyError,
-				},
+			resource: &anypb.Any{
+				TypeUrl: version.V3RouteConfigURL,
+				Value:   []byte{1, 2, 3, 4},
 			},
 			wantErr: true,
 		},
 		{
-			name: "empty resource list",
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			name:      "v2 routeConfig resource",
-			resources: []*anypb.Any{v2RouteConfig},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v2RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v2ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
+			name:     "v3 routeConfig resource",
+			resource: v3RouteConfig,
+			wantName: v3RouteConfigName,
+			wantUpdate: RouteConfigUpdate{
+				VirtualHosts: []*VirtualHost{
+					{
+						Domains: []string{uninterestingDomain},
+						Routes: []*Route{{Prefix: newStringP(""),
+							WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
+							ActionType:       RouteActionRoute}},
 					},
-					Raw: v2RouteConfig,
-				}},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			name:      "v2 routeConfig resource wrapped",
-			resources: []*anypb.Any{testutils.MarshalAny(&v2xdspb.Resource{Resource: v2RouteConfig})},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v2RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v2ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
+					{
+						Domains: []string{ldsTarget},
+						Routes: []*Route{{Prefix: newStringP(""),
+							WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
+							ActionType:       RouteActionRoute}},
 					},
-					Raw: v2RouteConfig,
-				}},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			name:      "v3 routeConfig resource",
-			resources: []*anypb.Any{v3RouteConfig},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v3RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v3RouteConfig,
-				}},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			name:      "v3 routeConfig resource wrapped",
-			resources: []*anypb.Any{testutils.MarshalAny(&v3discoverypb.Resource{Resource: v3RouteConfig})},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v3RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v3RouteConfig,
-				}},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			name:      "multiple routeConfig resources",
-			resources: []*anypb.Any{v2RouteConfig, v3RouteConfig},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v3RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v3RouteConfig,
-				}},
-				v2RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v2ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v2RouteConfig,
-				}},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusACKed,
-				Version: testVersion,
-			},
-		},
-		{
-			// To test that unmarshal keeps processing on errors.
-			name: "good and bad routeConfig resources",
-			resources: []*anypb.Any{
-				v2RouteConfig,
-				testutils.MarshalAny(&v3routepb.RouteConfiguration{
-					Name: "bad",
-					VirtualHosts: []*v3routepb.VirtualHost{
-						{Domains: []string{ldsTarget},
-							Routes: []*v3routepb.Route{{
-								Match: &v3routepb.RouteMatch{PathSpecifier: &v3routepb.RouteMatch_ConnectMatcher_{}},
-							}}}}}),
-				v3RouteConfig,
-			},
-			wantUpdate: map[string]RouteConfigUpdateErrTuple{
-				v3RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v3RouteConfig,
-				}},
-				v2RouteConfigName: {Update: RouteConfigUpdate{
-					VirtualHosts: []*VirtualHost{
-						{
-							Domains: []string{uninterestingDomain},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-						{
-							Domains: []string{ldsTarget},
-							Routes: []*Route{{Prefix: newStringP(""),
-								WeightedClusters: map[string]WeightedCluster{v2ClusterName: {Weight: 1}},
-								ActionType:       RouteActionRoute}},
-						},
-					},
-					Raw: v2RouteConfig,
-				}},
-				"bad": {Err: cmpopts.AnyError},
-			},
-			wantMD: UpdateMetadata{
-				Status:  ServiceStatusNACKed,
-				Version: testVersion,
-				ErrState: &UpdateErrorMetadata{
-					Version: testVersion,
-					Err:     cmpopts.AnyError,
 				},
+				Raw: v3RouteConfig,
+			},
+		},
+		{
+			name:     "v3 routeConfig resource wrapped",
+			resource: testutils.MarshalAny(&v3discoverypb.Resource{Resource: v3RouteConfig}),
+			wantName: v3RouteConfigName,
+			wantUpdate: RouteConfigUpdate{
+				VirtualHosts: []*VirtualHost{
+					{
+						Domains: []string{uninterestingDomain},
+						Routes: []*Route{{Prefix: newStringP(""),
+							WeightedClusters: map[string]WeightedCluster{uninterestingClusterName: {Weight: 1}},
+							ActionType:       RouteActionRoute}},
+					},
+					{
+						Domains: []string{ldsTarget},
+						Routes: []*Route{{Prefix: newStringP(""),
+							WeightedClusters: map[string]WeightedCluster{v3ClusterName: {Weight: 1}},
+							ActionType:       RouteActionRoute}},
+					},
+				},
+				Raw: v3RouteConfig,
 			},
-			wantErr: true,
 		},
 	}
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			opts := &UnmarshalOptions{
-				Version:   testVersion,
-				Resources: test.resources,
-			}
-			update, md, err := UnmarshalRouteConfig(opts)
+			name, update, err := unmarshalRouteConfigResource(test.resource)
 			if (err != nil) != test.wantErr {
-				t.Fatalf("UnmarshalRouteConfig(%+v), got err: %v, wantErr: %v", opts, err, test.wantErr)
+				t.Errorf("unmarshalRouteConfigResource(%s), got err: %v, wantErr: %v", pretty.ToJSON(test.resource), err, test.wantErr)
+			}
+			if name != test.wantName {
+				t.Errorf("unmarshalRouteConfigResource(%s), got name: %s, want: %s", pretty.ToJSON(test.resource), name, test.wantName)
 			}
 			if diff := cmp.Diff(update, test.wantUpdate, cmpOpts); diff != "" {
-				t.Errorf("got unexpected update, diff (-got +want): %v", diff)
-			}
-			if diff := cmp.Diff(md, test.wantMD, cmpOptsIgnoreDetails); diff != "" {
-				t.Errorf("got unexpected metadata, diff (-got +want): %v", diff)
+				t.Errorf("unmarshalRouteConfigResource(%s), got unexpected update, diff (-got +want): %v", pretty.ToJSON(test.resource), diff)
 			}
 		})
 	}
 }
 
 func (s) TestRoutesProtoToSlice(t *testing.T) {
+	sm, _ := matcher.StringMatcherFromProto(&v3matcherpb.StringMatcher{MatchPattern: &v3matcherpb.StringMatcher_Exact{Exact: "tv"}})
 	var (
 		goodRouteWithFilterConfigs = func(cfgs map[string]*anypb.Any) []*v3routepb.Route {
 			// Sets per-filter config in cluster "B" and in the route.
@@ -1205,7 +941,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 									{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}, TypedPerFilterConfig: cfgs},
 									{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 								},
-								TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 							}}}},
 				TypedPerFilterConfig: cfgs,
 			}}
@@ -1250,7 +985,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 									{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 									{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 								},
-								TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 							}}}},
 			}},
 			wantRoutes: []*Route{{
@@ -1290,7 +1024,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 								}}}},
 				},
 			},
@@ -1336,7 +1069,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 								}}}},
 				},
 			},
@@ -1355,6 +1087,51 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 			}},
 			wantErr: false,
 		},
+		{
+			name: "good with string matcher",
+			routes: []*v3routepb.Route{
+				{
+					Match: &v3routepb.RouteMatch{
+						PathSpecifier: &v3routepb.RouteMatch_SafeRegex{SafeRegex: &v3matcherpb.RegexMatcher{Regex: "/a/"}},
+						Headers: []*v3routepb.HeaderMatcher{
+							{
+								Name:                 "th",
+								HeaderMatchSpecifier: &v3routepb.HeaderMatcher_StringMatch{StringMatch: &v3matcherpb.StringMatcher{MatchPattern: &v3matcherpb.StringMatcher_Exact{Exact: "tv"}}},
+							},
+						},
+						RuntimeFraction: &v3corepb.RuntimeFractionalPercent{
+							DefaultValue: &v3typepb.FractionalPercent{
+								Numerator:   1,
+								Denominator: v3typepb.FractionalPercent_HUNDRED,
+							},
+						},
+					},
+					Action: &v3routepb.Route_Route{
+						Route: &v3routepb.RouteAction{
+							ClusterSpecifier: &v3routepb.RouteAction_WeightedClusters{
+								WeightedClusters: &v3routepb.WeightedCluster{
+									Clusters: []*v3routepb.WeightedCluster_ClusterWeight{
+										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
+										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
+									},
+								}}}},
+				},
+			},
+			wantRoutes: []*Route{{
+				Regex: func() *regexp.Regexp { return regexp.MustCompile("/a/") }(),
+				Headers: []*HeaderMatcher{
+					{
+						Name:        "th",
+						InvertMatch: newBoolP(false),
+						StringMatch: &sm,
+					},
+				},
+				Fraction:         newUInt32P(10000),
+				WeightedClusters: map[string]WeightedCluster{"A": {Weight: 40}, "B": {Weight: 60}},
+				ActionType:       RouteActionRoute,
+			}},
+			wantErr: false,
+		},
 		{
 			name: "query is ignored",
 			routes: []*v3routepb.Route{
@@ -1370,7 +1147,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 								}}}},
 				},
 				{
@@ -1486,14 +1262,13 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 0}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 0}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 0},
 								}}}},
 				},
 			},
 			wantErr: true,
 		},
 		{
-			name: "totalWeight is nil in weighted clusters action",
+			name: "The sum of all weighted clusters is more than uint32",
 			routes: []*v3routepb.Route{
 				{
 					Match: &v3routepb.RouteMatch{
@@ -1504,35 +1279,14 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 							ClusterSpecifier: &v3routepb.RouteAction_WeightedClusters{
 								WeightedClusters: &v3routepb.WeightedCluster{
 									Clusters: []*v3routepb.WeightedCluster_ClusterWeight{
-										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 20}},
-										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 30}},
+										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: math.MaxUint32}},
+										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: math.MaxUint32}},
 									},
 								}}}},
 				},
 			},
 			wantErr: true,
 		},
-		{
-			name: "The sum of all weighted clusters is not equal totalWeight",
-			routes: []*v3routepb.Route{
-				{
-					Match: &v3routepb.RouteMatch{
-						PathSpecifier: &v3routepb.RouteMatch_Prefix{Prefix: "/a/"},
-					},
-					Action: &v3routepb.Route_Route{
-						Route: &v3routepb.RouteAction{
-							ClusterSpecifier: &v3routepb.RouteAction_WeightedClusters{
-								WeightedClusters: &v3routepb.WeightedCluster{
-									Clusters: []*v3routepb.WeightedCluster_ClusterWeight{
-										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 50}},
-										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 20}},
-									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
-								}}}},
-				},
-			},
-			wantErr: true,
-		},
 		{
 			name: "unsupported cluster specifier",
 			routes: []*v3routepb.Route{
@@ -1587,7 +1341,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 30}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 20}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 50},
 								}}}},
 				},
 			},
@@ -1628,7 +1381,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 								}},
 							HashPolicy: []*v3routepb.RouteAction_HashPolicy{
 								{PolicySpecifier: &v3routepb.RouteAction_HashPolicy_FilterState_{FilterState: &v3routepb.RouteAction_HashPolicy_FilterState{Key: "io.grpc.channel_id"}}},
@@ -1686,7 +1438,6 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 										{Name: "B", Weight: &wrapperspb.UInt32Value{Value: 60}},
 										{Name: "A", Weight: &wrapperspb.UInt32Value{Value: 40}},
 									},
-									TotalWeight: &wrapperspb.UInt32Value{Value: 100},
 								}},
 							HashPolicy: []*v3routepb.RouteAction_HashPolicy{
 								{PolicySpecifier: &v3routepb.RouteAction_HashPolicy_Header_{Header: &v3routepb.RouteAction_HashPolicy_Header{HeaderName: ":path"}}},
@@ -1762,7 +1513,7 @@ func (s) TestRoutesProtoToSlice(t *testing.T) {
 	defer func() { envconfig.XDSRingHash = oldRingHashSupport }()
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			got, _, err := routesProtoToSlice(tt.routes, nil, nil, false)
+			got, _, err := routesProtoToSlice(tt.routes, nil)
 			if (err != nil) != tt.wantErr {
 				t.Fatalf("routesProtoToSlice() error = %v, wantErr %v", err, tt.wantErr)
 			}
@@ -1875,7 +1626,7 @@ func (s) TestHashPoliciesProtoToSlice(t *testing.T) {
 	defer func() { envconfig.XDSRingHash = oldRingHashSupport }()
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			got, err := hashPoliciesProtoToSlice(tt.hashPolicies, nil)
+			got, err := hashPoliciesProtoToSlice(tt.hashPolicies)
 			if (err != nil) != tt.wantErr {
 				t.Fatalf("hashPoliciesProtoToSlice() error = %v, wantErr %v", err, tt.wantErr)
 			}
